
create database DutchLyrics
on primary 
  (name = DutchLyrics_file, filename = N'C:\Files\DutchLyrics\DutchLyrics_File.mdf' ),
filegroup FSStorage contains filestream 
  ( name = DutchLyrics_fs_file, filename = N'C:\Files\DutchLyrics\DutchLyrics_FS' )
with filestream ( 
    non_transacted_access = full,
    directory_name = N'DutchLyrics' )
go

use DutchLyrics
go

-- create a filetable
create table Songs as filetable
with (filetable_directory = N'Songs')
go

-- in general, find Windows Search Properties here: http://msdn.microsoft.com/en-us/library/dd561977(v=VS.85).aspx
-- or here http://msdn.microsoft.com/en-us/library/dd368864(VS.85).aspx#generic
-- add a search property list
CREATE SEARCH PROPERTY LIST DocumentProperties;
GO
ALTER SEARCH PROPERTY LIST DocumentProperties
   ADD 'Title'
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 2, 
      PROPERTY_DESCRIPTION = 'System.Title - Title of the item.' );
GO
ALTER SEARCH PROPERTY LIST DocumentProperties
    ADD 'Author'
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 4, 
      PROPERTY_DESCRIPTION = 'System.Author - Author or authors of the item.' );
GO
ALTER SEARCH PROPERTY LIST DocumentProperties 
    ADD 'Tags'
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 5, 
      PROPERTY_DESCRIPTION = 'System.Keywords - Keywords (Tags) of the item.' );
GO

-- 
SELECT * FROM sys.registered_search_properties;
SELECT * FROM sys.registered_search_property_lists;

SELECT name, prop.*
FROM sys.registered_search_properties prop
JOIN sys.registered_search_property_lists list
ON prop.property_list_id = list.property_list_id

-- Support Dutch (1043) in FTS, but not yet Semantic Index
SELECT * 
from sys.fulltext_languages
where name like 'Dutch';

SELECT * 
from sys.fulltext_semantic_languages
where name like 'Dutch';

-- create FTS catalog
CREATE FULLTEXT CATALOG ft AS DEFAULT -- WITH ACCENT_SENSITIVITY ON
GO

-- FTS index needs a unique index, add it
CREATE UNIQUE INDEX SongsFt ON DutchLyrics..Songs(stream_id)
GO

-- Create FTS index

CREATE FULLTEXT INDEX ON DutchLyrics..Songs
	(file_stream TYPE COLUMN file_type LANGUAGE 1043)
	KEY INDEX SongsFt ON ft
	WITH SEARCH PROPERTY LIST = DocumentProperties
GO

-- check population status
SELECT fulltextcatalogproperty('ft', 'populatestatus')
GO

-- Traditional FTS
-- princess
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'prinsesse');

-- More traditional FTS
-- beautiful, fair
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'schoon');

-- shoe
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'schoen');

SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'prinsesse OR schoon');

SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'prinsesse AND schoen');

SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'prinsesse AND (schoen OR liefste)');

-- New NEAR operator
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'NEAR(("prinsesse", "schoen"), 5, FALSE)');

-- New NEAR operator
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'NEAR(("prinsesse", "schoen"), 5, TRUE)');

SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(file_stream, 'NEAR(("schoen", "prinsesse"), 5, TRUE)');

-- Property-based search
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(PROPERTY(file_stream, 'Author'), 'Antwerps');
GO

SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Songs
	WHERE CONTAINS(PROPERTY(file_stream, 'Title'), 'liedekens');
GO


-- Let's go search for words to look for
SELECT * FROM sys.dm_fts_index_keywords(DB_ID('DutchLyrics'),OBJECT_ID('dbo.Songs'))
order by document_count desc;

select * from sys.dm_fts_index_keywords(DB_ID('DutchLyrics'),OBJECT_ID('dbo.Songs')) k
where k.display_term not in 
(select s.stopword from sys.fulltext_system_stopwords s where s.language_id = 1043)
order by document_count desc

SELECT * FROM sys.dm_fts_index_keywords_by_document(DB_ID('DutchLyrics'),OBJECT_ID('dbo.Songs'))

SELECT * FROM sys.dm_fts_index_keywords_by_property(DB_ID('DutchLyrics'),OBJECT_ID('dbo.Songs'))



-- Have to repopulate the index
ALTER FULLTEXT INDEX ON DutchLyrics..Songs
START FULL POPULATION;

-- Internals 
SELECT * FROM sys.internal_tables;


-- drop database when done
drop database DutchLyrics